"use strict";

module.exports = function(babel) {
    var t = babel.types;
    return new babel.Transformer("babel-ui5-plugin", {
        Program : {
            exit(node, parent, scope, conf)
            {
                this.state.moduleFormatter.hasLocalExports = false;
            }
        },

        ImportDeclaration(node)
        {
            let path = node.source.value;
            let name = null;
            if (!node.specifier)
            {
                var parts = path.split("/");
                name = parts[parts.length - 1];
            }
            else
            {
                name = node.specifiers[0].local.name;
            }

            if (!this.state.ui5)
            {
                this.state.ui5 = { imports: [] };
            }
            this.state.ui5.imports.push({
                name: name,
                path: path
            });

            if (node.leadingComments)
            {
                this.state.ui5.leadingComments = node.leadingComments;
            }

            this.dangerouslyRemove();
        },

        ExportDeclaration(node, parent)
        {
            const depPaths = [];
            const depClasses = [];
            if (this.state.ui5 && this.state.ui5.imports)
            {
                this.state.ui5.imports.forEach((imp) => {
                    depPaths.push(imp.path);
                    depClasses.push(imp.name);
                });
            }

            const defineCallArgs = [
                t.arrayExpression(depPaths.map(t.literal)),
                t.functionExpression(null, depClasses.map(t.identifier), t.blockStatement([
                    t.expressionStatement(t.literal("use strict")),
                    t.returnStatement(node.declaration)
                ]))
            ];

            const defineCall = t.callExpression(t.identifier("\n\nsap.ui.define"), defineCallArgs);

            let leadingComments = [];
            if (this.state.ui5 && this.state.ui5.leadingComments)
            {
                leadingComments = this.state.ui5.leadingComments;
            }

            var globals = [ "sap" ];
            if (this.scope.globals)
            {
                for (let key in this.scope.globals)
                {
                    if (["arguments", "parseInt", "parseFloat", "undefined", "sap"].indexOf(key) === -1 && globals.indexOf(key) === -1)
                    {
                        globals.push(key);
                    }
                }
            }

            [
                "eslint eol-last: 0",
                "eslint no-underscore-dangle: 0",
                "global " + globals.join(", ")
            ].forEach(comment => {
                leadingComments.push({
                    type: "CommentBlock",
                    value: " " + comment + " "
                });
            });

            node.leadingComments = leadingComments;

            return defineCall;
        }
    });
};
